import bpkio_cli.utils.prompt as prompt
import click
import cloup
from bpkio_api.models import AdInsertionServiceIn, SourceType
from bpkio_cli.core.app_context import AppContext


def create_ad_insertion_service_command():
    # COMMAND: CREATE
    @cloup.command(help="Create a (simple) Ad Insertion service")
    @click.pass_obj
    def create(obj: AppContext):
        create_ad_insertion_service_via_prompts(obj)

    return create


def create_ad_insertion_service_via_prompts(obj: AppContext):
    all_sources = obj.api.sources.list()

    # Ask for the source
    sources = [
        s for s in all_sources if s.type in (SourceType.LIVE, SourceType.ASSET_CATALOG)
    ]
    sources = sorted(sources, key=lambda s: s.id, reverse=True)
    sources = obj.cache.sort_resources_by_most_recently_accessed(sources)
    choices = [
        dict(value=s.id, name=f"({s.id})  {s.name}  [{s.type.value}]") for s in sources
    ]
    source_id = prompt.fuzzy(
        message="Source",
        choices=choices,
    )
    source_type = next(s for s in sources if s.id == source_id).type

    # Then ask for the ad server
    ad_sources = [s for s in all_sources if s.type == SourceType.AD_SERVER]
    ad_sources = sorted(ad_sources, key=lambda s: s.id, reverse=True)
    ad_sources = obj.cache.sort_resources_by_most_recently_accessed(ad_sources)
    choices = [dict(value=s.id, name=f"({s.id})  {s.name}") for s in ad_sources]
    ad_source_id = prompt.fuzzy(
        message="Ad Server",
        choices=choices,
    )

    # Then ask for the gap filler
    slate_sources = [s for s in all_sources if s.type == SourceType.SLATE]
    slate_sources = sorted(slate_sources, key=lambda s: s.id, reverse=True)
    slate_sources = obj.cache.sort_resources_by_most_recently_accessed(slate_sources)
    choices = [dict(value=s.id, name=f"({s.id})  {s.name}") for s in slate_sources]
    choices = [dict(value=None, name="-- No gap filler --")] + choices
    slate_source_id = prompt.fuzzy(
        message="Gap Filler",
        choices=choices,
    )

    # Ask for the type of ad insertion (optionally)
    if source_type in (SourceType.ASSET, SourceType.ASSET_CATALOG):
        insertion_type = "vodAdInsertion"
    else:
        choices = [
            dict(value="liveAdPreRoll", name="Live Pre-Roll"),
            dict(value="liveAdReplacement", name="Live Ad Replacment"),
        ]
        insertion_type = prompt.select(
            message="Ad Insertion Type",
            choices=choices,
        )

    # Specific options
    max_duration = None
    if insertion_type == "liveAdPreRoll":
        # Ask for the duration
        max_duration = prompt.text(
            message="Max duration (in seconds)",
            validate=lambda result: result.isdigit(),
            invalid_message="The duration must be a number.",
        )

    # Ask for transcoding profile
    profiles = obj.api.transcoding_profiles.list()
    profiles = obj.cache.sort_resources_by_most_recently_accessed(profiles)
    choices = [dict(value=s.id, name=f"({s.id})  {s.name}") for s in profiles]
    # ... add a "none" one
    choices = [dict(value=None, name="-- No transcoding --")] + choices
    transcoding_profile_id = prompt.fuzzy(
        message="Transcoding Profile",
        choices=choices,
    )

    # Ask for other options
    with_transcoding = transcoding_profile_id is not None

    name = prompt.text(
        message="Name for the service",
        validate=lambda result: len(result) > 0,
        invalid_message="The name cannot be empty.",
    )

    # Create the service object
    service = AdInsertionServiceIn(
        name=name, source=dict(id=source_id), enableAdTranscoding=with_transcoding
    )
    ad_insertion = dict(adServer=dict(id=ad_source_id))
    if max_duration is not None:
        ad_insertion["maxDuration"] = int(max_duration)
    if slate_source_id is not None:
        ad_insertion["gapFiller"] = dict(id=slate_source_id)

    setattr(
        service,
        insertion_type,
        ad_insertion,
    )
    if with_transcoding:
        # setattr(service, "transcodingProfile", dict(id=transcoding_profile_id))
        service.transcodingProfile = dict(id=transcoding_profile_id)

    service_out = obj.api.services.ad_insertion.create(service)

    obj.response_handler.treat_single_resource(service_out)
